library('bnlearn')
# read data files

bi_gene<- read.delim("./data2/biclustering_mRNA_3module.csv",header=FALSE)
bi_pro <- read.delim("./data2/biclustering_protein_3module.csv",header=FALSE)
gene_list<- as.vector(unlist(read.delim("./data2/GBM_genelist.txt",header=FALSE)))
pro_list <- as.vector(unlist(read.delim("./data2/GBM_protein_list.txt",header=FALSE)))
gexp <- read.delim("./data2/GBM_mRNA_tumor.txt",header=FALSE)
pexp <- read.delim("./data2/GBM_protein_tumor.txt",header=FALSE)
mirna_exp <- read.delim("./data2/GBM_mirna_tumor.txt",header=FALSE)
mirna_list<- as.vector(unlist(read.delim("./data2/GBM_miRNA_list.txt",header=FALSE)))
gm_mat <- read.delim("./data2/GBM_mirna_protein_mRNA_mat.txt",header=FALSE)


module_num <- nrow(bi_gene)
module_num2 <- nrow(bi_pro)
module_gene_list <- list()
module_pro_list <- list()

attach(module_gene_list)

for(i in 1:module_num){
  module_genes <- unlist(strsplit(as.character(bi_gene[i,]),","))
  module_genes <- module_genes[!module_genes==""]
  module_gene_list[[i]] <- module_genes
}

detach(module_gene_list)


attach(module_pro_list)

for(i in 1:module_num2){
  module_pros <- unlist(strsplit(as.character(bi_pro[i,]),","))
  module_pros <- module_pros[!module_pros==""]
  module_pro_list[[i]] <- module_pros
}

detach(module_pro_list)


# use thres% microRNAs as candiates

module_mirna_list <- list()

bic_score <- -1000000
bic_prev_score <- -1000000

gm_mat <- abs(gm_mat)
thres <- quantile(as.vector(unlist(gm_mat)),0.80,na.rm=TRUE)

for(i in 1:length(module_gene_list)){

  order_idx <- 1
  selected_mirna <- NULL
 
  module_genes <- module_gene_list[[i]]
  module_pros <- module_pro_list[[i]]

  cor_list <- NULL
  idx <- NULL
  idx2 <- NULL
  sum_idx <- NULL
  mirna_idx <- NULL
  bic_score <- 0
  module_genes_len <- length(module_genes)


  # find index for gene_matrix and sub correlation matrix

  for(j in 1:length(module_genes)){
   idx <- c(idx,which(gene_list == module_genes[j]))
  }

  for(j in 1:length(module_pros)){
   idx2 <- c(idx2,which(pro_list == module_pros[j]))
  }

  sum_idx <- idx2 


  # find sub gene expression
  sub_gexp <- gexp[idx,]
  sub_pexp <- pexp[idx2,]
  sub_gpexp <- rbind(sub_gexp,sub_pexp)

  # find sub microrna expression

  sub_cor_mat <- rbind(gm_mat[idx,],gm_mat[sum_idx,])
  sub_cor_mat_mean <- apply(sub_cor_mat,2,mean,na.rm=TRUE)
  mirna_order <- order(-sub_cor_mat_mean)

  sub_mirna_idx <- which(sub_cor_mat_mean > thres)
  sub_mirna_exp <- mirna_exp[sub_mirna_idx,]
  sub_mirna_list <- mirna_list[sub_mirna_idx]

  # find sub correlation matrix

  sub_cor_mat <- sub_cor_mat[,sub_mirna_idx]


  sub_exp_mat <- t(rbind(sub_gpexp, sub_mirna_exp))
  sub_exp_mat_colnames <- c(module_genes,module_pros,mirna_list[sub_mirna_idx])


  graph = empty.graph(sub_exp_mat_colnames)

  # convert matrix into dataframe
  colnames(sub_exp_mat) <- sub_exp_mat_colnames
  rownames(sub_exp_mat) <- 1:nrow(sub_exp_mat)
  sub_exp_mat <- data.frame(sub_exp_mat)
  colnames(sub_exp_mat) <- sub_exp_mat_colnames

  sub_exp_mat[is.na(sub_exp_mat)] <- 0
	
  bic_score <- score(graph, sub_exp_mat, type="bic-g")


  # threshold is updating until bic_score is not improved

  order_idx <- 1
  while(TRUE){

  print("Execute")

  prev_bic_score <- bic_score
  sel_mirna <- mirna_list[mirna_order[order_idx]]

  if(sub_cor_mat_mean[mirna_order[order_idx]] < thres)
    break

  selected_gene_idx <- NULL
  selected_pro_idx <- NULL

  for(j in 1:length(module_genes)){
    graph = set.arc(graph, sel_mirna,module_genes[j])
    bic_score <- score(graph, sub_exp_mat, type="bic-g")
    if(bic_score > prev_bic_score)
      selected_gene_idx <- c(selected_gene_idx,j)
    graph = drop.arc(graph, sel_mirna,module_genes[j])
  }

  for(j in 1:length(module_pros)){
    graph = set.arc(graph, sel_mirna,module_pros[j])
    bic_score <- score(graph, sub_exp_mat, type="bic-g")
    if(bic_score > prev_bic_score)
      selected_pro_idx <- c(selected_pro_idx,j)
    graph = drop.arc(graph, sel_mirna,module_pros[j])
  }




  # there is no improvement
  if(length(selected_gene_idx) == 0)
     break

  if(length(selected_pro_idx) == 0)
     break

  for(j in 1:length(selected_gene_idx))
  {
     graph = set.arc(graph,sel_mirna,module_genes[selected_gene_idx[j]])
  }


    for(j in 1:length(selected_pro_idx))
  {
     graph = set.arc(graph,sel_mirna,module_pros[selected_pro_idx[j]])
  }


  bic_score <- score(graph, sub_exp_mat, type="bic-g")

   if(bic_score < prev_bic_score)
     break
    selected_mirna <- c(selected_mirna,sel_mirna)
    order_idx <- order_idx + 1
  }
  module_mirna_list[[i]] <- unique(selected_mirna)
  print(i)
}

for(i in 1:length(module_mirna_list))
{

  if(length(module_mirna_list[[i]]) < 2){
    idx <- c(idx,i)
    next
  }
  print(i)
  module_mirnas <- module_mirna_list[[i]]
  
  module_mirnas <- unique(module_mirnas)
  module_mirna_list[[i]] <- module_mirnas
}

for(i in 1:length(module_mirna_list))
{
  if(length(module_mirna_list[[i]]) < 2)
    next


  write.table(t(as.character(module_gene_list[[i]])),"./result2/biclustering_mRNA_3module_withmirna.csv",sep=",",qmethod="double",append=TRUE,col.names=FALSE,row.names=FALSE)
  write.table(t(as.character(module_pro_list[[i]])),"./result2/biclustering_protein_3module_withmirna.csv",sep=",",qmethod="double",append=TRUE,col.names=FALSE,row.names=FALSE)
  write.table(t(as.character(module_mirna_list[[i]])),"./result2/biclustering_mirna_3module_withmirna.csv",sep=",",qmethod="double",append=TRUE,col.names=FALSE,row.names=FALSE)
}





binding <- list()

attach(binding)

for(i in 1:length(module_mirna_list))
{
  if(length(module_mirna_list[[i]]) < 2)
    next


  a_gene <- t(as.character(module_gene_list[[i]]))
  b_pro <- t(as.character(module_mirna_list[[i]]))

  binding[[i]] <- c(a_gene,b_pro)


  
  }

detach(binding)




for(i in 1:length(module_mirna_list))
{


if(length(module_mirna_list[[i]]) < 2)
    next


   write.table(t(as.character(binding[[i]])),"./result2/biclustering_mRNA_protein_3module.csv",sep=",",qmethod="double",append=TRUE,col.names=FALSE,row.names=FALSE)
  
  }

